var async = require('async');
var fs = require('fs');
var path = require('path');
var events = require('events');

var _ = require('lodash');

var installer = require('./installer');
var system = require('./utils/system');
var glob = require('glob');

var abcRoot = system.getUserHome('.abc');

function parsePackages (packages) {

    if (!packages) {
        return [];
    }

    if (_.isString(packages)) {
        packages = packages.split(' ');
    }

    if (!_.isArray(packages)) {
        packages = [];
    }
    return packages.map(function(name){
        return 'generator-' + name
    });
}

var spawnOptions = {
    cwd: abcRoot
};

var GeneratorManager = new events.EventEmitter();

_.extend(GeneratorManager, {

    install: function(packages, callback) {
        packages = parsePackages(packages);
        return installer.run('install', packages, null, spawnOptions, callback);
    },

    update: function (packages, callback) {
        packages = parsePackages(packages);
        return installer.run('update', packages, null, spawnOptions, callback);
    },

    remove: function (packages, callback) {
        packages = parsePackages(packages);
        installer.run('rm', packages, null, spawnOptions, callback);
    },

    exists: function (type) {
        var pkgRoot = path.join(abcRoot, 'node_modules', 'generator-' + type);
        if (!fs.existsSync(pkgRoot)) {
            return null;
        }

        var pkg = JSON.parse(fs.readFileSync(path.join(pkgRoot, 'package.json')));

        return pkg;
    },

    list: function (callback) {

        var options = {
            cwd: abcRoot
        };

        glob("node_modules/generator-*/", options, function (er, files) {
            async.map(files, function(file, callback) {

                var name = file.match(/generator-([^/]+)/)[1];
                var root = path.join(abcRoot, file);
                var pkgFile = path.join(root, 'package.json');
                try {
                    var pkg = JSON.parse(fs.readFileSync(pkgFile));
                } catch (e) {
                    callback(null, {
                        root: root,
                        name: name,
                        pkg: null,
                        status: 'error'
                    });
                    return;
                }


                callback(null, {
                    root: root,
                    name: name,
                    pkg: pkg
                });
            }, callback);
        })
    }
});

module.exports = GeneratorManager;