var util = require('util');
var path = require('path');
var spawn = require('child_process').spawn;
var dargs = require('dargs');
var _ = require('lodash');

var childManager = require('./child-manager');

var Runner = require('./runner');

module.exports = GruntRunner;


function GruntRunner (options) {
    Runner.apply(this);
    this.root = options.root;
    this.gruntPath = path.join(this.root, 'node_modules', 'grunt/');
    this.childGrunt = path.join(__dirname, 'child-grunt.js');
}

util.inherits(GruntRunner, Runner);

GruntRunner.prototype.run = function run(tasks, options, callback) {
    var args = arguments;

    process.nextTick(function(){
        this._run.apply(this, args);
    }.bind(this));

};

GruntRunner.prototype._run = function _run(tasks, options, callback){

    var env = _.clone(process.env);

    env['LOCAL_GRUNT'] = this.gruntPath;

    if (tasks && _.isString(tasks)) {
        tasks = tasks.split(' ');
    } else if (!_.isArray(tasks)){
        tasks = [];
    }

    var args = [this.childGrunt].concat(tasks).concat(dargs(options));

    this.child = spawn(process.execPath, args, {
        env: env,
        cwd: this.root,
        silent: true
    });

    this.emit('child:start', this.child.pid);

    this.child.on('exit', function(code) {
        if (code != 0) {
            this.emit('child:end');
            return;
        }

        this.emit('child:end');

    }.bind(this));

    this.child.on('error', function(error) {
         this.emit('child:error', error);
    }.bind(this));

    this.watchChild(this.child);

    callback && callback(null, {
        id: childManager.add(this.child)
    });
};
