function ProjectCtrl ($scope, socket, $location, yeoman, grunt, navbar, projectCache) {
    navbar.active('project');
    socket.init($scope);

    yeoman.init($scope, socket);
    grunt.init($scope, socket);

    $scope.title = 'loading...';
    $scope.loaded = false;
    $scope.stdLog = '';
    $scope.root = $location.search().root;
    $scope.alerts = [];
    $scope.status = [];
    $scope.isBlank = false;
    $scope.showYeoman = false;

    $scope.newPage = {
        name: 'foo',
        version: '1.0'
    }

    $scope.appCache = projectCache.get($scope.root);

    $scope.closeAlert = function(index) {
        $scope.alerts.splice(index, 1);
    };

    if (!$scope.root) {
        $location.url('/');
        return;
    }

    socket.emit('cd', {
        root: $scope.root
    }, function (data) {
        if (data.status !== 'ok') {
            alert(data.message);
        }
    });

    $scope.$watch(function() {
        if ($scope.appCache) {
            projectCache.put($scope.root, $scope.appCache);
        }
    });

    socket
        .on('ready', function() {
            $scope.loaded = true;

            socket.emit('info', function (data) {
                $scope.title = data.name;
                $scope.info = data;
                $scope.isBlank = !$scope.info.abc;
                $scope.yeoman.type = data.abc ? data.abc.type : '';
            });
        });

    socket.on('yeoman:ready', function(){
        socket.emit('yeoman:ui', function(data){
            $scope.UI = data;
        });
    });

    socket.on('yeoman:scan', function(data){
        $scope.app = data;
    });

    socket.on('action:error', function (error) {
        $scope.alerts.push({
            type: 'error',
            msg: error.message
        });
    });

    socket.on('action:pingback', function () {
        alert('ping back');
    });

    socket.on('action:status:update', handleStatus.bind($scope));

    $scope.$on('$destroy', function () {
        console.log('scope destorying..')
    });






    // Scope Function
    $scope.exit = function () {
        exitCurrent();
    };

    $scope.runYeoman = function () {

        $scope.stdLog = '';

        socket.emit('yeoman:run', {
            type: $scope.yeoman.type,
            sub: $scope.yeoman.sub

        });
    };

    $scope.runYeomanSub = function (sub) {

        if (!sub) {
            $scope.alerts.push({
                type: 'info',
                msg: "runYeomanSub: no sub"
            });
            return;
        }

        $scope.stdLog = '';

        socket.emit('yeoman:run', {
            sub: sub
        });
    };



    $scope.exitYeoman = function() {
        socket.emit('yeoman:exit');
    };

    socket.on('yeoman:start', function() {
        $scope.showYeoman = true;
    });

    $scope.socketEmit = function (eventName, data) {
        socket.emit(eventName, data);
    };

    // Helper Function
    function exitCurrent() {
        socket.emit('exit', function () {
            $location.url('/');
        });
    }
}