APP.factory('projectCache', function() {

    var prefix = 'projectCache_';

    var cache = {};

    var throttledPut = _.throttle(function(key, val) {
        var str = JSON.stringify({
            val: val
        });

        if (cache[key] === str) {
            return;
        }
        localStorage.setItem(prefix + key, str);
    }, 600);

    return {
        put: throttledPut,

        get: function(key) {
            console.log('get key')

            var v = localStorage.getItem(prefix + key);
            var value;
            if (v) {
                try {
                    value = JSON.parse(v).val;
                } catch (e) {
                    value = null;
                }
            }

            return value;
        }
    };

});

APP.factory('socket', function ($rootScope, notify) {
    var socket = io.connect();



    function initEvent() {
        socket
            .on('disconnect', function() {
                $('#socket').
                    removeClass('status-connected').
                    addClass('status-disconnected');
            })
            .on('connect', function () {
                $('#socket').
                    removeClass('status-disconnected').
                    addClass('status-connected');
            })

            .on('error', function(data) {
                notify({
                    title: data.title || '出错了！',
                    text: data.message || '没有错误信息',
                    type: 'error'
                });
            })

            .on('action:error', function(data) {
                notify({
                    title: 'ABC',
                    text: data.message,
                    type: 'error'
                });
            })

    }

    var socket2 = {

        init: function () {
            socket.removeAllListeners();

            initEvent();

            $rootScope.systemOpen = function(file) {
                socket.emit('open', file);
            };

            $rootScope.cliDialogs = [];

            socket2.on('process:new', function(data) {

                var pid = data.pid;

                var p = {
                    pid: pid,
                    title: data.title,
                    log: '',
                    show: true,
                    done: false,
                    dialogOption: {
                        backdropFade: true,
                        dialogFade:true,
                        backdropClick: true,
                        keyboard: false
                    }
                };

                $rootScope.cliDialogs.push(p);

                socket2.on('process:stdout:'+pid, function(out){
                    p.log += out;
                });

                socket2.on('process:exit:' + pid, function(){

                    p.show = false;
                    p.done = true;

//                    var index = _.findIndex($rootScope.cliDialogs, function(item){
//                        return item.pid === pid;
//                    });
//                    if (index > -1) {
//                        $rootScope.cliDialogs.splice(index ,1);
//                    }

                    socket.removeListener('process:stdout:'+pid);
                    socket.removeListener('process:exit'+pid);
                });

            });
        },

        on: function (eventName, callback) {
            socket.on(eventName, function () {
                var args = arguments;
                console.log('   socket event:', eventName, args[0]);
                $rootScope.$apply(function () {
                    callback.apply(socket, args);
                });
            });
            return this;
        },

        emit: function (eventName, data, callback) {
            var args = [eventName];


            if (_.isFunction(data)) {
                callback = data;
            } else if (data) {
                args.push(data);
            }


            console.log('    socket emit:', eventName);

            function cb() {
                var args = arguments;
                console.log('socket callback:', eventName, args[0]);
                $rootScope.$apply(function () {
                    if (callback) {
                        callback.apply(socket, args);
                    }
                });
            }

            args.push(cb);

            socket.emit.apply(socket, args);
            return this;
        }
    };

    initEvent();

    return socket2;
});

APP.factory('query', function() {
    function getQueryVariable(str) {

        if (!str) {
            return {};
        }

        var obj = {};
        var query = str;

        var vars = query.split('&');

        for (var i = 0; i < vars.length; i++) {
            var pair = vars[i].split('=');
            obj[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1])
        }
        return obj;
    }

    return {
        parse: function(str) {
            return getQueryVariable(str)
        }
    }
});

APP.factory('navbar', function() {
    return {
        active: function(name) {
            var $topbar =  $('#navbar')
            $topbar.children('.active').removeClass('active');
            $topbar.children('li[data-name="' + name + '"]').addClass('active');

        }
    }
});


APP.factory('yeoman', function(query, $timeout, notify) {
    return {
        init: function($scope, socket) {

            socket.on('yeoman:start', function() {
                $scope.yeoman.log = '';
                $scope.yeoman.show = true;
                $scope.yeoman.running = true;
            });

            socket.on('yeoman:error', function(err){
                notify({
                    title: 'Yeoman Error',
                    text: err.message || 'Unknow Error',
                    type: 'error'
                });
            });

            socket.on('yeoman:stdout', function(str) {
                $scope.yeoman.log += str;
            });

            socket.on('yeoman:end', function (err) {
                console.log(err);
                $scope.yeoman.running = false;

                if (err) {
                    notify({
                        title: 'Yeoman End with Error',
                        text: err.message || 'Unknow Error',
                        type: 'error'
                    });
                    return;
                }

                notify({
                    title: 'Yeoman run successful!',
                    text: 'No error was detected, sir',
                    type: 'success'
                });

                $timeout(function() {
                    $scope.yeoman.resolvable.show = false;
                    $scope.yeoman.show = false;
                }, 500);
            });

            socket.on('yeoman:resolvable', function(res) {
                var resolvable = $scope.yeoman.resolvable;

                if (res.type === 'prompts') {
                    var prompts = res.data.map(function(prompt){
                        prompt.value = prompt.default;
                        return prompt;
                    });
                    resolvable.prompts = prompts;

                }



                if (res.type === 'conflict') {
                    resolvable.conflict = res.conf;
                }


                resolvable.show = true;

            });

            $scope.yeoman = {
                show: false,
                running: false,
                log: '',

                dialogOption: {
                    backdropFade: true,
                    dialogFade:true,
                    backdropClick: false,
                    keyboard: false,
                    dialogClass: 'modal modal-yeoman'
                },

                resolvable: {
                    show: false,
                    conflict: null,
                    prompt: null,

                    dialogOption: {
                        keyboard: false,
                        backdrop: true,
                        backdropClick: false,
                        dialogClass: 'modal modal-resolvable'
                    },

                    resolve : function () {

                        var resolvable = $scope.yeoman.resolvable;


                        if (!resolvable) {
                            return;
                        }

                        if (resolvable.prompts) {

                            var props = _(resolvable.prompts).map(function(prompt) {
                                return [prompt.name, prompt.value]
                            }).object().value();

                            socket.emit('yeoman:resolve', {
                                type: 'prompt',
                                props: props
                            });

                            resolvable.prompts = null;
                        }

                        if (resolvable.conflict) {
                            var method = _.find(resolvable.conflict.options, 'checked').name;

                            if (!method) {
                                return;
                            }
                            socket.emit('yeoman:resolve', {
                                type: 'conflict',
                                filepath: resolvable.conflict.filepath,
                                method: method
                            });

                            resolvable.conflict = null;
                        }

                        resolvable.show = false;
                    }
                },

                run: function(ev) {
                    var data = {};

                    if (ev) {
                        var et = $(ev.target);
                        ev.preventDefault();

                        var type = et.attr('data-type');
                        var sub = et.attr('data-sub');
                        var options = et.attr('data-options');
                        var args = et.attr('data-args');

                        if (type) {
                            data.type = type.trim();
                        }

                        if (sub) {
                            data.sub = sub.trim();
                        }

                        if (options) {
                            data.options = query.parse(options);
                        }

                        if (args) {
                            data.args = args.split(' ')
                        }

                        socket.emit('yeoman:run', data);
                    }


                },

                exit: function() {
                    socket.emit('yeoman:exit');
                    $scope.yeoman.show = false;
                }


            }

        }
    }
});

APP.factory('grunt', function(query, $timeout, notify) {

    return {
        init: function($scope, socket){

            $scope.grunt = {
                show: false,
                pid: null,
                log: '',
                running: false,
                dialogOption: {
                    backdropFade: true,
                    dialogFade:true,
                    backdropClick: false
                },
                run: function(ev) {
                    var data = {};

                    if (ev) {
                        var et = $(ev.target);

                        var tasks = et.attr('data-tasks');
                        var options = query.parse(et.attr('data-attrs'));

                        if (tasks) {
                            data.tasks = tasks
                        }

                        if (options) {
                            data.options = options
                        }
                    }


                    socket.emit('grunt:run', data);
                },

                exit: function() {

                    if ($scope.grunt.running) {
                        socket.emit('grunt:exit', $scope.grunt.pid);
                    }

                    $scope.grunt.show = false;
                }
            };

            socket.on('grunt:start', function(id) {
                $scope.grunt.log = '';
                $scope.grunt.running = true;
                $scope.grunt.show = true;
                $scope.grunt.pid = id;
            });

            socket.on('yeoman:error', function(err){
                notify({
                    title: 'Grunt Error',
                    text: err.message || 'Unknow Error',
                    type: 'error'
                });
            });

            socket.on('grunt:end', function() {

                $scope.grunt.running = false;

                notify({
                    title: 'Grunt ',
                    text: '执行完毕 ',
                    type: 'success'
                });

                $timeout(function () {
                    $scope.grunt.show = false;
                }, 500);
            });

            socket.on('grunt:stdout', function(str) {
                $scope.grunt.log += str;
            });


        }
    }
});


APP.factory('notify', function() {
    $.pnotify.defaults.history = false;
    $.pnotify.defaults.styling = "bootstrap";
    return $.pnotify;

});